/*  Copyright (C) 2023 Chair of Aircraft Design, Technical University Munich
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file reportGenerator.h
 * @brief Report generator.
 * @version 1.0.0
 * @date 2024-12-20
 */

#ifndef SRC_REPORT_GENERATOR_H_
#define SRC_REPORT_GENERATOR_H_

 /* === Includes === */
#include <algorithm>
#include <cctype>
#include <cstddef>
#include <filesystem>
#include <moduleBasics/module.h>
#include <runtimeInfo/runtimeInfo.h>
#include <stdexcept>
#include <string>
#include <vector>
#include <aircraftGeometry2/geometry/surface.h>
#include <aixml/node.h>
#include <aircraftGeometry2/processing/transform.h>

namespace geometry
{
    /* === Typedefs for the geometry components === */
    using Fuselage = geom2::MultisectionSurface<geom2::PolygonSection>;
    using Wing = geom2::MultisectionSurface<geom2::AirfoilSection>;
    using Nacelle = geom2::MultisectionSurface<geom2::PolygonSection>;
    using Pylon = geom2::MultisectionSurface<geom2::AirfoilSection>;

    /* === Classes === */
    /**
     * @struct Aircraft
     * @brief Groups the aircraft geometry components.
     */
    struct Aircraft
    {
        std::vector<Fuselage> fuselages;
        std::vector<Wing> wings;
        std::vector<Wing> empennage;
        std::vector<Nacelle> nacelle;
        std::vector<Pylon> pylon;
    };
}
/* === Classes === */
class ReportGenerator : public Module
{
public:
    ReportGenerator(const int argc, char** argv, const std::string& toolName, const std::string& toolVersion)
        : Module(argc, argv, toolName, toolVersion)
    {
    }

    ReportGenerator(const std::string& toolName, const std::string& toolVersion, const std::filesystem::path& rtConfigXML)
        : Module(toolName, toolVersion, rtConfigXML)
    {
    }

    /**
     * @brief Destroy the ReportGenerator module instance.
     */
    ~ReportGenerator() = default;

    /* === Populate the module template methods === */
    void initialize() override;
    void run() override;
    void update() override;
    void report() override;
    void save() override;

private:

    /* === Methods === */

    /**
     * @brief Writes the content of the report sections
     *
     * @throws 
     */
    void write_dropdown_content();

    /**
     * @brief Searches for the TLARs and writes them to the report
     * @note This is a recursive function
     *
     * @param TLAR The root node
     */
    void get_all_TLAR(const auto TLAR);

    /**
     * @brief Formats the provided element
     *
     * @param element String element to be formatted
     * @return element Formatted string element
     */
    auto format_element(std::string element)->std::string;

    /**
     * @brief Writes the links to reports to the report
     * @note Only the reporting path is searched,
     * reports are assumed to have _report.html in their
     * titles
     *
     * @param reporting_path the path where the reports are located
     */
    void add_links(std::filesystem::path reporting_path);

    /**
     * @brief Writes the clustered aircraft design paramters to
     * the report
     * @note Clusters are pre-defined
     *
     */
    void add_clusters();

    /**
     * @brief Writes the value and unit, or the value of a
     * node to the report
     *
     * @param element The path to the node
     * @return label The string element that can be written to the 
     * report
     */
    auto generic_writer(std::string element)->std::string;

    /**
     * @brief Creates the aircraft geometry, the aircraft
     * geometry mesh and writes it to an stl file
     * @note The aircraft geometry and mesh is set to null
     * if one or more elements of the aircraft are missing
     *
     */
    void create_aircraft_geometry();

    std::shared_ptr<node> aircraft_xml;                                  /** [-] The aircraft xml data. */
    std::shared_ptr<node> configuration_xml;                             /** [-] The configuration xml data. */
    std::vector<node*> TLARs;                                            /** [-] Top Level Aircraft Requirements. */
    std::fstream htmlcontent;                                            /** [-] The template file for the report. */
    std::fstream html_temp;                                              /** [-] The report file. */
    std::filesystem::path reporting_path;                                /** [-] Path to the reports. */
    std::shared_ptr<geometry::Aircraft> aircraft_geometry;               /** [-] The aircraft geometry. */
    geom2::Mesh aircraft_mesh;                                           /** [-] The aircraft mesh. */
};
#endif // SRC_REPORT_GENERATOR_H_